<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @author Aybat Duyshokov <duyshokov@gmail.com>
 * @package ow_plugins.friends.controllers
 * @since 1.0
 */
class FRIENDS_CTRL_List extends OW_ActionController
{

    private $params;

    public function __construct()
    {
        parent::__construct();
    }

    public function index($params)
    {
        
        $this->params = $params;

        $language = OW::getLanguage();

        $userService = BOL_UserService::getInstance();

        $page = (!empty($_GET['page']) && intval($_GET['page']) > 0 ) ? $_GET['page'] : 1;

        $rpp = 30;

        $first = ($page - 1) * $rpp;
        $count = $rpp;

        $case = $this->getCase();

        if( $case == 'user-friends' )
        {
            $this->setPageHeading(OW::getLanguage()->text('friends', 'user_friends_page_title', array('user'=>$params['user'])));
        }
        else
        {
            $this->setPageHeading(OW::getLanguage()->text('friends', 'friendship_management_area'));
        }

        $this->setPageHeadingIconClass('ow_ic_user');

        $this->assign('case', $case);

        list($list, $itemCount) = $this->getInfo($first, $count);

        $this->addComponent('menu', $this->getMenu());

        $this->addComponent('paging', new BASE_CMP_Paging($page, ceil($itemCount / $rpp), 5));

        $idList = array();

        $userList = array();

        foreach ( $list as $dto )
        {
            $userList[] = array(
                'dto' => $dto
            );

            $idList[] = $dto->getId();
        }

        $usernameList = array();
        $displayNameList = array();
        $avatarList = array();

        $questionList = array();
        $onlineInfo = array();

        $qs = array();

        $qBdate = BOL_QuestionService::getInstance()->findQuestionByName('birthdate', 'sex');

        if ( $qBdate->onView )
            $qs[] = 'birthdate';

        $qSex = BOL_QuestionService::getInstance()->findQuestionByName('sex');

        if ( $qSex->onView )
            $qs[] = 'sex';

        if ( !empty($idList) )
        {
            $displayNameList = $userService->getDisplayNamesForList($idList);
            $usernameList = $userService->getUserNamesForList($idList);
            $avatarList = BOL_AvatarService::getInstance()->getAvatarsUrlList($idList);

            $questionList = BOL_QuestionService::getInstance()->getQuestionData($idList, $qs);

            if ( $case != 'online' )
            {
                $onlineInfo = BOL_UserService::getInstance()->findOnlineStatusForUserList($idList);
            }
        }

        $this->assign('questionList', $questionList);

        $this->assign('usernameList', $usernameList);
        $this->assign('avatarList', $avatarList);
        $this->assign('displayNameList', $displayNameList);
        $this->assign('onlineInfo', $onlineInfo);

        $this->assign('list', $userList);
    }

    private function getInfo( $first, $count )
    {
        $service = FRIENDS_BOL_Service::getInstance();
        $userService = BOL_UserService::getInstance();

        $userId = OW::getUser()->getId();

        switch ( $this->getCase() )
        {
            case 'friends':
                $idList = $service->findFriendIdList($userId, $first, $count, 'friends');

                return array(
                    $userService->findUserListByIdList($idList),
                    $service->countFriends($userId)
                );

            case 'sent-requests':

                $idList = $service->findFriendIdList($userId, $first, $count, 'sent-requests');

                return array(
                    $userService->findUserListByIdList($idList),
                    $service->count($userId, null, FRIENDS_BOL_Service::STATUS_PENDING, FRIENDS_BOL_Service::STATUS_IGNORED)
                );

            case 'got-requests':

                $idList = $service->findFriendIdList($userId, $first, $count, 'got-requests');

                return array(
                    $userService->findUserListByIdList($idList),
                    $service->count(null, $userId, FRIENDS_BOL_Service::STATUS_PENDING)
                );
                
            case 'user-friends':
            
                $user = BOL_UserService::getInstance()->findByUsername($this->params['user']);
                $userId = $user->getId();

                $idList = $service->findFriendIdList($userId, $first, $count, 'friends');

                return array(
                    $userService->findUserListByIdList($idList),
                    $service->countFriends($userId)
                );                
        }

        return array(array(), 0);
    }

    private function getMenu()
    {
        $language = OW::getLanguage();

        $case = $this->getCase();

        $userId = OW::getUser()->getId();

        $item[0] = new BASE_MenuItem();

        $count = FRIENDS_BOL_Service::getInstance()->countFriends($userId);

        $item[0]->setLabel($language->text('friends', 'friends_tab', array('count' => ($count > 0) ? "({$count})" : '')))
            ->setKey('1')
            ->setUrl(OW::getRouter()
                ->urlForRoute('friends_list'))
            ->setOrder(1)
            ->setIconClass('ow_ic_clock');

        $case = $this->getCase();

        if ( $case == 'friends' )
        {
            $item[0]->setActive(true);
        }
//--
        $item[1] = new BASE_MenuItem();

        $count = FRIENDS_BOL_Service::getInstance()->count($userId, null, FRIENDS_BOL_Service::STATUS_PENDING, FRIENDS_BOL_Service::STATUS_IGNORED);

        $item[1]->setLabel($language->text('friends', 'sent_requests_tab', array('count' => ($count > 0) ? "({$count})" : '')))
            ->setKey('2')
            ->setUrl(OW::getRouter()
                ->urlForRoute('friends_sent_requests'))
            ->setOrder(2)
            ->setIconClass('ow_ic_push_pin');

        if ( $case == 'sent-requests' )
        {
            $item[1]->setActive(true);
        }
//--
        $item[2] = new BASE_MenuItem();

        $count = FRIENDS_BOL_Service::getInstance()->count(null, $userId, FRIENDS_BOL_Service::STATUS_PENDING);

        $item[2]->setLabel($language->text('friends', 'got_requests_tab', array('count' => ($count > 0) ? "({$count})" : '')))
            ->setKey('2')
            ->setUrl(OW::getRouter()
                ->urlForRoute('friends_got_requests'))
            ->setOrder(2)
            ->setIconClass('ow_ic_push_pin');

        if ( $case == 'got-requests' )
        {
            $item[2]->setActive(true);
        }
//--		
        $menu = new BASE_CMP_ContentMenu($item);

        
        
        return $menu->setVisible($case != 'user-friends');
    }

    private function getCase()
    {
        if ( count($_GET) )
        {
            $arr = explode('?', OW::getRequest()->getRequestUri());
        }

        if (
            (!empty($arr)
            && in_array(
                trim($arr[0]),
                array(
                    OW::getRouter()->uriForRoute('friends_list'),
                    OW::getRouter()->uriForRoute('friends_list') . '/'
                )
            )
            )
            || OW::getRequest()->getRequestUri() == OW::getRouter()->uriForRoute('friends_list')
        )
        {
            return 'friends';
        }

        if ( false !== strstr(OW::getRequest()->getRequestUri(), OW::getRouter()->uriForRoute('friends_sent_requests')) )
        {
            return 'sent-requests';
        }

        if ( false !== strstr(OW::getRequest()->getRequestUri(), OW::getRouter()->uriForRoute('friends_got_requests')) )
        {
            return 'got-requests';
        }

        if(!empty($this->params['user']) &&
            false !== strstr(OW::getRequest()->getRequestUri(), OW::getRouter()->uriForRoute('friends_user_friends', array('user'=>$this->params['user']))) 
        )
        {
            return 'user-friends';
        }
    }
}