<?php

/**
 * This software is intended for use with Oxwall Free Community Software http://www.oxwall.org/ and is
 * licensed under The BSD license.

 * ---
 * Copyright (c) 2009, Skalfa LLC
 * All rights reserved.

 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, this list of conditions and
 *  the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *  the following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *  - Neither the name of the Skalfa LLC nor the names of its contributors may be used to endorse or promote products
 *  derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Feed component
 *
 * @author Sergey Kambalin <greyexpert@gmail.com>
 * @package ow_plugins.newsfeed.components
 * @since 1.0
 */
class NEWSFEED_CMP_Feed extends OW_Component
{
    private $feed = array();
    
    /**
     * 
     * @var NEWSFEED_BOL_Service
     */
    protected $service;
    
    protected $onlineUserId;
    
    protected $moderatorMode;
    
    protected $configs = array();
    
    public function __construct( array $feed )
    {
        parent::__construct();
        
        $this->configs = OW::getConfig()->getValues('newsfeed');
        $this->feed = $feed;
        
        $this->service = NEWSFEED_BOL_Service::getInstance();
        
        $this->onlineUserId = OW::getUser()->isAuthenticated() ? OW::getUser()->getId() : false;
        $this->moderatorMode = $this->onlineUserId ? OW::getUser()->isAuthorized('newsfeed') : false;
    }
    
    private function extractUserIdList( $actionList )
    {
        $out = array();
        
        foreach ( $actionList as $action )
        {
            /* @var $action NEWSFEED_CLASS_Action */
            $out[] = $action->getUserId();
        }
        
        return $out;
    }
    
    private function getListForView()
    {
        $userIds = $this->extractUserIdList($this->feed);
        
        $usersInfo = array(
            'avatars' => BOL_AvatarService::getInstance()->getAvatarsUrlList($userIds),
            'urls' => BOL_UserService::getInstance()->getUserUrlsForList($userIds),
            'names' => BOL_UserService::getInstance()->getDisplayNamesForList($userIds)
        );
        
        $out = array();
        foreach ( $this->feed as $action )
        {
            /* @var $action NEWSFEED_CLASS_Action */
            $updateTime = $action->getUpdateTime();
            $sectionKey = date('dmY', $updateTime);

            if ( empty($out[$sectionKey]) )
            {
                $out[$sectionKey] = array(
                    'date' => UTIL_DateTime::formatDate($updateTime, true),
                    'list' => array()
                );
            }
            
            $out[$sectionKey]['list'][$action->getId()] = $this->getItemForView($action, $usersInfo);            
        }
        
        return $out;
    } 
    
    private function getItemForView( NEWSFEED_CLASS_Action $action, $usersInfo = array() )
    {
        $view = $action->getDataValue('view');
        $view = empty($view) ? array() : $view;
        $view = array_merge(array(
            'iconClass' => 'ow_ic_add',
            'class' => '',
            'style' => ''
        ), $view);
        
        $currentUri = OW::getRequest()->getRequestUri();
        $removeUrl = OW::getRequest()->buildUrlQueryString(OW::getRouter()->urlFor('NEWSFEED_CTRL_Feed', 'removeFeedItem'), array(
            'actionId' => $action->getId(),
            'backUri' => $currentUri));
         
        $removeUrl = $this->onlineUserId == $action->getUserId() || $this->moderatorMode 
                ? $removeUrl
                : false;

        $userNameEmbed = '<a href="' . $usersInfo['urls'][$action->getUserId()] . '">' . $usersInfo['names'][$action->getUserId()] . '</a>';
        $line = str_replace('[$user]', $userNameEmbed, $action->getDataValue('line'));
        $string = str_replace('[$user]', $userNameEmbed, $action->getDataValue('string'));
        $content = str_replace('[$user]', $userNameEmbed, $action->getDataValue('content'));
        
        $item = array(
            'id' => $action->getId(),
            'view' => $view,
            'toolbar' => $action->getDataValue('toolbar'),
            'string' => $string,
            'content' => $content,
            'entityType' => $action->getEntity()->type,
            'entityId' => $action->getEntity()->id,
            'id' => $action->getId(),
            'createTime' => UTIL_DateTime::formatDate($action->getCreateTime()),
            'updateTime' => $action->getUpdateTime(),
            
            'user' => array(
                'id' => $action->getUserId(),
                'avatarUrl' => $usersInfo['avatars'][$action->getUserId()],
                'url' => $usersInfo['urls'][$action->getUserId()],
                'name' => $usersInfo['names'][$action->getUserId()]
            ),
            'removeUrl' => $removeUrl,
                
            'line' => $line,
            'comments' => false,
            'likes' => false
        );
        
        $features = $action->getDataValue('features');
        $features = $features === null ? array('comments', 'likes') : $features;
        
        if ( in_array('comments', $features) )
        {
            $authGroup = $action->getPluginKey();
            $authActionDto = BOL_AuthorizationService::getInstance()->findAction($action->getPluginKey(), 'add_comment');
            
            if ( $authActionDto === null )
            {
                $authGroup = 'newsfeed';
            }

            $commentsParams = new BASE_CommentsParams($authGroup, $action->getEntity()->type);
            $commentsParams->setEntityId($action->getEntity()->id);
            $commentsParams->setDisplayType(BASE_CommentsParams::DISPLAY_TYPE_BOTTOM_FORM_WITH_PARTIAL_LIST_AND_MINI_IPC);
            $commentsParams->setCommentCountOnPage($this->configs['comments_count']);
            $commentCmp = new BASE_CMP_Comments($commentsParams);
            $commentTemplate = OW::getPluginManager()->getPlugin('newsfeed')->getCmpViewDir() . 'comments.html';
            $commentCmp->setTemplate($commentTemplate);
            $item['comments']['cmp'] = $commentCmp->render();
            $item['comments']['count'] = BOL_CommentService::getInstance()->findCommentCount($action->getEntity()->type, $action->getEntity()->id);
            $item['comments']['button'] = OW::getUser()->isAuthorized($authGroup, 'add_comment') && !$item['comments']['count'];
        }
        
        if ( $this->configs['allow_likes'] && in_array('likes', $features) )
        {
           $likes = $this->service->findEntityLikes($action->getEntity()->type, $action->getEntity()->id);
           
           $userLiked = false;
           foreach ( $likes as $like )
           {
                if ( $like->userId == OW::getUser()->getId() )
                {
                    $userLiked = true;
                }
           }
           
           $likeCmp = new NEWSFEED_CMP_Likes($action->getEntity()->type, $action->getEntity()->id, $likes);
           $item['likes']['count'] = count($likes);
           $item['likes']['button'] = OW::getUser()->isAuthenticated() && !$userLiked && $action->getUserId() != OW::getUser()->getId();
           $item['likes']['cmp'] = $likeCmp->render();
           
           $script = UTIL_JsGenerator::newInstance()
                ->setVariable(array('window', 'OW_NEWSFEED_LikesResponderUrl'), OW::getRouter()->urlFor('NEWSFEED_CTRL_Feed', 'ajaxLike'));
                
           OW::getDocument()->addScriptDeclaration($script->generateJs());
        }

        return $item;
    }
    
    public function render()
    {
        $this->assign('feed', $this->getListForView());
        
        return parent::render();
    }
}